package com.persagy.apm.diagnose.utils;

import java.beans.BeanInfo;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.util.CollectionUtils;

import com.alibaba.fastjson.JSONObject;


/**
 * 数值、集合处理工具类
 *
 * @author cuixubin
 */
public class DataUtils {

    /**
     * 对象转Double类型
     *
     * @param object
     * @return
     */
    public static Double parseDouble(Object object) {
        if (object == null) {
            return null;
        }
        if (object instanceof String) {
            return Double.valueOf(object.toString());
        }
        if (object instanceof Integer) {
            return ((Integer) object).doubleValue();
        }
        if (object instanceof Long) {
            return ((Long) object).doubleValue();
        }
        if (object instanceof Double) {
            return (Double) object;
        }
        if (object instanceof BigDecimal) {
            return ((BigDecimal) object).doubleValue();
        }
        return (Double) object;
    }

    /**
     * 设置小数精度
     *
     * @param obj
     * @param count 精确的小数位数
     * @return
     */
    public static Double setDecimalScale(Double obj, int count) {
        if (obj == null) {
            return null;
        }
        BigDecimal bd = new BigDecimal(obj);
        return bd.setScale(count, BigDecimal.ROUND_HALF_UP).doubleValue();
    }

    /**
     * 将Double类型的数字向下取整
     *
     * @param num
     * @return
     */
    public static String getInt(Object num) {
        if (num == null) {
            return null;
        }
        try {
            Double dob = Double.valueOf(num + "");
            DecimalFormat df = new DecimalFormat("#.#");
            return df.format(dob);
        } catch (Exception e) {
            return num + "";
        }
    }

    /**
     * BigDecimal精确计算
     *
     * @param dividend
     * @param divisor
     * @param decimalScale 小数点精确范围
     * @return
     * @throws Exception
     */
    public static <T extends Number> Double divide(T dividend, T divisor, Integer decimalScale) throws Exception {
        if (dividend == null || divisor == null) {
            return null;
        }
        if (divisor.doubleValue() == 0) {
            return null;
        }
        if (decimalScale == null) {
            decimalScale = DECIMAL_SCALE;
        }
        BigDecimal dividendDecimal = new BigDecimal(dividend.toString());
        BigDecimal divisorDecimal = new BigDecimal(divisor.toString());
        return dividendDecimal.divide(divisorDecimal, decimalScale, BigDecimal.ROUND_HALF_UP).doubleValue();
    }

    private static final int DECIMAL_SCALE = 12;
    public static <T extends Number> Double divide(T dividend, T divisor) throws Exception {
        return divide(dividend,divisor,DECIMAL_SCALE);
    }

    /*private static Double round(Double value, int scale) {
        if (value == null) {
            return null;
        }
        BigDecimal bd = new BigDecimal(value);
        bd = bd.setScale(scale, BigDecimal.ROUND_HALF_UP);
        return bd.doubleValue();
    }*/

    public static Double subtract(Double minuend, Double subtrahend) {
        if (minuend == null && subtrahend == null) {
            return null;
        }
        if (minuend == null) {
            return -subtrahend;
        }
        if (subtrahend == null) {
            return minuend;
        }
        return minuend - subtrahend;
    }

    public static Double plus(Double data1, Double data2) {
        if (data1 == null && data2 == null) {
            return null;
        }
        if (data1 == null) {
            return data2;
        }
        if (data2 == null) {
            return data1;
        }
        return data1 + data2;
    }

    public static Double plus(Double... dataList) {
        Double sum = null;
        for (Double data : dataList) {
            sum = plus(sum, data);
        }
        return sum;
    }



    public static Integer getInt(Double data) {
        if (data == null) {
            return null;
        }
        return (int) (data + 0.5);
    }

    public static String formatData(Double energyData) {
        if (energyData == null) {
            return "--";
        }
        double result;
        if (energyData < 1) {
            result = DataUtils.setDecimalScale(energyData, 3);
        } else if (energyData < 1000) {
            result = DataUtils.setDecimalScale(energyData, 1);
        } else {
            result = Math.round(energyData);
        }
        NumberFormat nf = NumberFormat.getInstance();
        nf.setGroupingUsed(false);
        String resultStr = nf.format(result);
        if (resultStr.indexOf(".") > 0) {
            //正则表达 去掉后面无用的零
            resultStr = resultStr.replaceAll("0+?$", "");
            //如小数点后面全是零则去掉小数点
            resultStr = resultStr.replaceAll("[.]$", "");
        }
        return resultStr;
    }

    public static void main(String[] args) throws Exception {
        System.out.println(formatData(0.4343));
    }

    public static Double formatDataNumber(Double energyData) {
        if (energyData == null) {
            return null;
        }
        if (energyData < 1) {
            return DataUtils.setDecimalScale(energyData, 3);
        } else if (energyData < 1000) {
            return DataUtils.setDecimalScale(energyData, 1);
        } else {
            return Math.round(energyData) * 1.0;
        }

    }

    @SuppressWarnings({ "unchecked", "rawtypes" })
	public static void sortList(List srcList, String sortField, boolean isAsc) throws Exception{
        if(CollectionUtils.isEmpty(srcList)){
            return;
        }
        Collections.sort(srcList, (o1, o2) -> {
            JSONObject object1 = JSONObject.parseObject(JSONObject.toJSONString(o1));
            JSONObject object2 = JSONObject.parseObject(JSONObject.toJSONString(o2));
            Object name1 = object1.get(sortField);
            Object name2 = object2.get(sortField);
            int coefficient;
            if (isAsc) {
                coefficient = 1;
            } else {
                coefficient = -1;
            }
            if (name1 != null && name2 != null) {
                if (name1 instanceof Double) {
                    return ((Double) name1).compareTo((Double) name2) * coefficient;
                } else if (name1 instanceof Integer) {
                    return ((Integer) name1).compareTo((Integer) name2) * coefficient;
                } else if (name1 instanceof BigDecimal) {
                    return ((BigDecimal) name1).compareTo((BigDecimal) name2) * coefficient;
                } else {
                    return ((String) name1).compareTo(((String) name2)) * coefficient;
                }
            } else if (name1 != null) {
                return coefficient;
            } else if (name2 != null) {
                return -coefficient;
            } else {
                return 0;
            }

        });
    }




    /**
     * 分位函数 算法参考http://www.cnblogs.com/gispathfinder/p/5770091.html
     *
     * @param dataList
     * @param location
     * @return
     * @throws Exception
     */
    public static Double quantileCompute(List<Double> dataList, Double location) throws Exception {

        if (CollectionUtils.isEmpty(dataList) || dataList.size() < 4) {
            return null;
        }
        List<Double> sortedDataList = new ArrayList<>();
        sortedDataList.addAll(dataList);
        sortList(sortedDataList);
        Double quantileLocation = (dataList.size() + 1) * location;
        int quantileInteger = quantileLocation.intValue();
        if (quantileInteger == 0) {
            if (sortedDataList.get(quantileInteger) != null) {
                return quantileLocation * sortedDataList.get(quantileInteger);
            }
        } else {
            if (sortedDataList.get(quantileInteger - 1) != null && sortedDataList.get(quantileInteger) != null) {
                return sortedDataList.get(quantileInteger - 1) + (quantileLocation - quantileInteger) * (sortedDataList.get(quantileInteger) - sortedDataList.get(quantileInteger - 1));
            }
        }
        return null;
    }

    public static void sortList(List<Double> energyList) {
        Collections.sort(energyList, (o1, o2) -> {
            if (o1 == null && o2 == null) {
                return 0;
            } else if (o1 == null) {
                return -1;
            } else if (o2 == null) {
                return 1;
            } else {
                return o1.compareTo(o2);
            }
        });
    }
}
